/*
 * DVAAPI.C
 *
 * Interface glue code to the Direct Video Access 
 * in ATI's Multimedia Video Drivers
 *
 * Copyright (C) December 1995, ATI Technologies Inc.
 *
 * [99/12/28 MS] - Now compiles under VC6 
 */

#include <windows.h>
#include <mmsystem.h>

#include "ATIVideo.h"
#include "parm.h"

static HINSTANCE hDllDVA = 0L;

/*
 * Capture functions group
 */
static DWORD	(WINAPI *pDVAGetCaps)(WORD)								= NULL;
static HDVA 	(WINAPI *pDVAMakeCapture)(WORD, LPRECT, DWORD)			= NULL;
static void		(WINAPI *pDVAFreeCapture)(HDVA)							= NULL;
static BOOL		(WINAPI *pDVACheckCapture)(HDVA)						= NULL;
static WORD		(WINAPI *pDVACapture)(HDVA, HPVOID)						= NULL;
static BOOL		(WINAPI *pDVASetCrop)(HDVA, LPRECT)						= NULL;
static WORD     (WINAPI *pDVAVfwSetCC)(WORD, BOOL, HWND, UINT, WORD)	= NULL;
static void		(WINAPI *pDVAShowConfig)(WORD)							= NULL;
/*
 * Property functions group
 */
static DWORD	(WINAPI *pDVAGetProperty)(WORD, WORD)					= NULL;
static void		(WINAPI *pDVASetProperty)(WORD, WORD, DWORD)			= NULL;
static BOOL		(WINAPI *pDVAEnumConnectors)(WORD, WORD, WORD far *)	= NULL;
/*
 * Tuner functions group
 */
static DWORD	(WINAPI *pDVATuneCaps)(WORD)							= NULL;
static DWORD	(WINAPI *pDVATuneChannels)(WORD)						= NULL;
static WORD		(WINAPI *pDVATuneScan)(WORD, LPVOID)					= NULL;
static WORD		(WINAPI *pDVATuneSetChannel)(WORD, WORD)				= NULL;
static WORD		(WINAPI *pDVATuneSetCable)(WORD, WORD)					= NULL;
static WORD		(WINAPI *pDVATuneSetStandard)(WORD, WORD)				= NULL;
static BOOL		(WINAPI *pDVATuneEnumFormat)(WORD, WORD, BOOL far *, WORD far *, LPBYTE)	= NULL;
static WORD		(WINAPI *pDVATuneFineTune)(WORD, WORD, BYTE)			= NULL;
static int		(WINAPI *pDVATuneSetList)( WORD, DWORD, BOOL) 			= NULL; 
static WORD		(WINAPI *pDVATuneGetLockStatus)( WORD, WORD)			= NULL;
static BOOL		(WINAPI *pDVATunerChannelSurfing)( WORD, BOOL) 			= NULL;
/*
 * Sound functions group
 */
static DWORD	(WINAPI *pDVAAudioCaps)(WORD)							= NULL;
static DWORD	(WINAPI *pDVAAudioSetStereoVolume)(WORD, WORD, WORD)	= NULL;
static DWORD	(WINAPI *pDVAAudioSetVolume)(WORD, WORD)				= NULL;
static DWORD	(WINAPI *pDVAAudioGetVolume)(WORD)						= NULL;
static DWORD	(WINAPI *pDVAAudioSetMix)(WORD, WORD)					= NULL;
static DWORD	(WINAPI *pDVAAudioSetSwitch)(WORD, WORD)				= NULL;
static BOOL		(WINAPI *pDVAAudioDetectStereo)(WORD)					= NULL;
static WORD		(WINAPI *pDVASndSetExternal)(WORD, BOOL)				= NULL;
static WORD		(WINAPI *pDVAAudioSetMute)(BOOL)						= NULL;
static DWORD	(WINAPI *pDVAAudioDetectStatus)(WORD)					= NULL;
static DWORD	(WINAPI *pDVAAudioSetChannel)(WORD,DWORD)				= NULL;


static void		(WINAPI *pDVAInit)(void)								= NULL;
static void		(WINAPI *pDVAFree)(void)								= NULL;
int ver = 1;			// Defines platform: 0 - WINNT or WIN3.1, 1 - WIN95


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

BOOL _stdcall DVALoad(HINSTANCE hInstance)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	LPSTR sretval=strdup("0123");  //VC6 modification

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_HANDLE, DTC_PTR, DTC_END};
	//char				szBuf [81];
	BOOL	    (WINAPI	*pDVAInitDll)(void)                         = NULL;    
	int nreturn ;

	hInstance=0;
	/* 
	 * Load the DLL
	 */
	hDllDVA = LoadLibrary ("DVA32.DLL");
	if (!hDllDVA) {
		DVAFree();
		return FALSE;
	}

	/* 
	 * Initialize the Dll by calling DVAInitDll(), if that function
         * is found in DLL (NT needs it)
	 */	
    pDVAInitDll	        = (BOOL(WINAPI	*)(void))GetProcAddress(hDllDVA, "DVAInitDll");
    if (pDVAInitDll)
        { // pDVAInitDll() found in the Dll, try to initialize Dll
          // pDVAInitDll() is optional in Dlls, but if exists, must be called 
        if ( !pDVAInitDll() )
            { // Failed to initialize Dll, unload it
             pDVAInitDll = NULL;
	     DVAFree();
	     return FALSE;
            }

        }


	// Be very careful here ! Make sure this matches the parameters to this method!
	// otherwise you will crash the 16 bit DLL
    Invoke16Func = (void (WINAPIV *) (short,unsigned long*,BYTE [],...)) GetProcAddress (hDllDVA,"Invoke16Func");
	if (!Invoke16Func) return 0;
	Invoke16Func(1,&ulError,parms,hInstance,sretval);

	//ver = 1;
	nreturn = (int)(sretval[0]);
	free(sretval) ;    //VC6 modification
	return nreturn ;
}

void _stdcall DVAFree(void)
{               
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;

	if (!ver) {
		if (pDVAFree) pDVAFree();

		// Remove references to DVA functions
		pDVAGetCaps					= NULL;
		pDVAMakeCapture				= NULL;
		pDVAFreeCapture				= NULL;
		pDVACheckCapture			= NULL;
		pDVACapture					= NULL;
		pDVASetCrop					= NULL; 
		pDVAShowConfig				= NULL;
		pDVAVfwSetCC    			= NULL;
		pDVAGetProperty				= NULL;
		pDVASetProperty				= NULL;
		pDVAEnumConnectors			= NULL;
		pDVATuneCaps				= NULL;
		pDVATuneChannels			= NULL;
		pDVATuneScan				= NULL;
		pDVATuneSetChannel			= NULL;
		pDVATuneSetCable			= NULL;
		pDVATuneSetStandard			= NULL;
		pDVATuneEnumFormat			= NULL;
		pDVATuneFineTune			= NULL;
		pDVATuneSetList				= NULL; 
		pDVATuneGetLockStatus		= NULL;
		pDVATunerChannelSurfing		= NULL;
		pDVAAudioCaps				= NULL;
		pDVAAudioSetStereoVolume	= NULL;
		pDVAAudioSetVolume			= NULL;
		pDVAAudioSetMute			= NULL;
		pDVAAudioGetVolume			= NULL;
		pDVAAudioSetMix				= NULL;
		pDVAAudioSetSwitch			= NULL;    
		pDVAAudioDetectStereo		= NULL;
		pDVASndSetExternal			= NULL;
		pDVAAudioDetectStatus		= NULL;
		pDVAAudioSetChannel			= NULL;
		// Unload the driver	

		FreeLibrary(hDllDVA);

		hDllDVA = NULL;
	} else {

		// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
		BYTE parms[] = {DTC_END};

		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (Invoke16Func)
			Invoke16Func(2,&ulError,parms);
		Invoke16Func = NULL;
		FreeLibrary (hDllDVA);
		hDllDVA = NULL;
	}
}

DWORD _stdcall DVAGetCaps(WORD wCard)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD,DTC_PTR, DTC_END};
	if (!ver) {
		if (!pDVAGetCaps) return 0;
		return pDVAGetCaps(wCard);
	} else {
		if (!Invoke16Func) return FALSE;
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		Invoke16Func(3,&ulError,parms,wCard,&retval);
		return (retval);
	}
}

HDVA _stdcall DVAMakeCapture(WORD wCard, LPRECT lprcImage, DWORD dwFormat)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_WORD,
					DTC_WORD,DTC_WORD,DTC_DWORD,DTC_PTR,DTC_END};
	if (!ver) {
		if (!pDVAMakeCapture) return 0;
		return pDVAMakeCapture(wCard, lprcImage, dwFormat);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(4,&ulError,parms,wCard,
				lprcImage->left,lprcImage->top,
				lprcImage->right,lprcImage->bottom,dwFormat,&retval);
		return (retval);
	}
}

void _stdcall DVAFreeCapture(HDVA hCapture)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_PTR,DTC_END};
	if (!ver)
		if (pDVAFreeCapture) pDVAFreeCapture(hCapture);
	else
		if (Invoke16Func) Invoke16Func(5,&ulError,parms,&hCapture);
}

BOOL _stdcall DVACheckCapture(HDVA hCapture)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	int retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_PTR,DTC_PTR,DTC_END};
	if (!ver) {
		if (!pDVACheckCapture) return 0;
		return pDVACheckCapture(hCapture);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(6,&ulError,parms,&hCapture,&retval);
		return (retval);
	}
}

WORD _stdcall DVACapture(HDVA hCapture, HPVOID hpNextBuffer)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_PTR,DTC_PTR,DTC_PTR,DTC_END};
	if (!ver) {
		if (!pDVACapture) return 0;
		return pDVACapture(hCapture, hpNextBuffer);
	} else {
		DWORD dwNextBuffer = (DWORD)hpNextBuffer;

		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(7,&ulError,parms,&hCapture,&dwNextBuffer,&retval);
		return (retval);
	}
}

BOOL _stdcall DVASetCrop(HDVA hCapture, LPRECT lprcCrop)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_DWORD,DTC_WORD,DTC_WORD,DTC_WORD,DTC_WORD,DTC_PTR,DTC_END};
	if (!ver) {
		if (!pDVASetCrop) return FALSE;
		return pDVASetCrop(hCapture, lprcCrop);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(8,&ulError,parms,hCapture,lprcCrop->left,lprcCrop->top,
			lprcCrop->right,lprcCrop->bottom,&retval);
		return (retval);
	}
}

void _stdcall DVASetProperty(WORD wCard, DVAProperty wProperty, DWORD dwNewValue)
{                              
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_DWORD, DTC_END};
	if (!ver) {
		if( !pDVASetProperty ) return;
		pDVASetProperty(wCard, (WORD)wProperty, dwNewValue);
	} else { 
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (Invoke16Func)
			Invoke16Func(9,&ulError,parms,wCard,wProperty,dwNewValue);
	}
}

DWORD _stdcall DVAGetProperty(WORD wCard, DVAProperty wProperty)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAGetProperty ) return 0;
		return pDVAGetProperty(wCard, (WORD)wProperty);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(10,&ulError,parms,wCard,wProperty,&retval);
		return (retval);
	}
}

BOOL _stdcall DVAEnumConnectors(WORD wCard, WORD wIndex, WORD far *lpwInfo)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAEnumConnectors ) return FALSE;
		return pDVAEnumConnectors(wCard, wIndex, lpwInfo);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(27,&ulError,parms,wCard,wIndex,lpwInfo,&retval);
		return (retval);
	}
}

DWORD _stdcall DVATuneCaps(WORD wCard)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneCaps ) return 0;
		return pDVATuneCaps(wCard);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(11,&ulError,parms,wCard,&retval);
		return (retval);
	}
}

DWORD _stdcall DVATuneChannels(WORD wCard)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneChannels ) return 0;
		return pDVATuneChannels(wCard);
	} else{
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(12,&ulError,parms,wCard,&retval);
		return (retval);
	}
}

WORD _stdcall DVATuneScan(WORD wCard, TUNERCALLBACK lpfnCallback)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_DWORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneScan ) return 0;
		return pDVATuneScan(wCard, lpfnCallback);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(13,&ulError,parms,wCard,(DWORD)lpfnCallback,&retval);
		return (retval);
	}
}

WORD _stdcall DVATuneSetChannel(WORD wCard, WORD wChannel)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneSetChannel ) return 0;
		return pDVATuneSetChannel(wCard, wChannel);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(14,&ulError,parms,wCard,wChannel,&retval);
		return (retval);
	}
}

WORD _stdcall DVATuneSetCable(WORD wCard, WORD bIsCable)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneSetCable ) return 0;
		return pDVATuneSetCable(wCard, bIsCable);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(15,&ulError,parms,wCard,bIsCable,&retval);
		return (retval);
	}
}

WORD _stdcall DVATuneSetStandard(WORD wCard, WORD wStandard)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneSetStandard ) return 0;
		return pDVATuneSetStandard(wCard, wStandard);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(16,&ulError,parms,wCard,wStandard,&retval);
		return (retval);
	}
}

BOOL _stdcall DVATuneEnumFormat(WORD wCard, WORD wIndex, BOOL far *lpfIsCable, WORD far *lpwStd, LPBYTE lpName)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_PTR, DTC_PTR, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneEnumFormat ) return FALSE;
		return pDVATuneEnumFormat(wCard, wIndex, lpfIsCable, lpwStd, lpName);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(17,&ulError,parms,wCard,wIndex,lpfIsCable,lpwStd,lpName,&retval);
		*lpfIsCable = LOWORD(*lpfIsCable);
		return (retval);
	}
}

WORD _stdcall DVATuneFineTune(WORD wCard, WORD wChannel, BYTE bOffset)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_BYTE, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneFineTune ) return 0;
		return pDVATuneFineTune(wCard, wChannel, bOffset);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(18,&ulError,parms,wCard,wChannel,bOffset,&retval);
		return (retval);
	}
}

int _stdcall DVATuneSetList( WORD wCard, DWORD lptfTunerFormatList, BOOL fCableOrAir)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_DWORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATuneFineTune) return 0;
		return( pDVATuneSetList( wCard, lptfTunerFormatList, fCableOrAir));
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(19,&ulError,parms,wCard,lptfTunerFormatList,fCableOrAir,&retval);
		return (retval);
	}
}

BOOL ATIAPI DVATunerChannelSurfing(WORD wCard, BOOL fSwitch)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVATunerChannelSurfing) return 0;
		return(pDVATunerChannelSurfing(wCard, fSwitch));
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(28,&ulError,parms,wCard,(WORD)fSwitch,&retval);
		return (retval);
	}
}


DWORD _stdcall DVAAudioCaps(WORD wCard)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAAudioCaps ) return 0;
		return pDVAAudioCaps(wCard);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(20,&ulError,parms,wCard,&retval);
		return (retval);
	}
}

DWORD _stdcall DVAAudioSetStereoVolume(WORD wCard, WORD wLVolume, WORD wRVolume)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAAudioSetStereoVolume ) return 0;
		return pDVAAudioSetStereoVolume(wCard, wLVolume, wRVolume);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(21,&ulError,parms,wCard,wLVolume,wRVolume,&retval);
		return (retval);
	}
}

DWORD _stdcall DVAAudioSetVolume(WORD wCard, WORD wVolume)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAAudioSetVolume ) return 0;
		return pDVAAudioSetVolume(wCard, wVolume);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(22,&ulError,parms,wCard,wVolume,&retval);
		return (retval);	
	}
}

BOOL _stdcall DVAAudioSetMute(BOOL bMute)
{
	unsigned long ulError;
	WORD retval = 0;

	BYTE parms[] = {DTC_WORD,DTC_PTR,DTC_END};
	if (!ver) {
		if( !pDVAAudioSetMute ) return 0;
		return pDVAAudioSetMute(bMute);
	} else {
		if (!Invoke16Func) return FALSE;
		Invoke16Func(37,&ulError,parms,(WORD)bMute,&retval);
		return retval;
	}
}

DWORD _stdcall DVAAudioGetVolume(WORD wCard)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAAudioGetVolume ) return 0;
		return pDVAAudioGetVolume(wCard);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(23,&ulError,parms,wCard,&retval);
		return (retval);
	}
}

DWORD _stdcall DVAAudioSetMix(WORD wCard, WORD wMix)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAAudioSetMix ) return 0;
		return pDVAAudioSetMix(wCard, wMix);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(24,&ulError,parms,wCard,wMix,&retval);
		return (retval);
	}
}

DWORD _stdcall DVAAudioSetSwitch(WORD wCard, WORD bSwitch)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	DWORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAAudioSetSwitch ) return 0;
		return pDVAAudioSetSwitch(wCard, bSwitch);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(25,&ulError,parms,wCard,bSwitch,&retval);
		return (retval);
	}
}

BOOL _stdcall DVAAudioDetectStereo(WORD wCard)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval=99;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_PTR, DTC_END};
	if (!ver) {
		if( !pDVAAudioDetectStereo ) return FALSE;
		return pDVAAudioDetectStereo(wCard);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (!Invoke16Func) return FALSE;
		Invoke16Func(29,&ulError,parms,wCard,&retval);
		return (BOOL)(retval);
	}
}

WORD _stdcall DVASndSetExternal(WORD wCard, BOOL fSwitch)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
    WORD retval = 99;
    BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_PTR,DTC_END};
	if (!ver) {
		if(!pDVASndSetExternal) return((WORD)-1);
		return(pDVASndSetExternal(wCard, fSwitch));
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
        if (!Invoke16Func) return ((WORD)-1);
		Invoke16Func(31,&ulError,parms,wCard,(WORD)fSwitch,&retval);
		return (retval);
	}
}

void _stdcall DVAShowConfig(WORD wCard)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD, DTC_END};
	if (!ver) {
		if (!pDVAShowConfig) return;
		pDVAShowConfig(wCard);
	} else {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		if (Invoke16Func)
			Invoke16Func(26,&ulError,parms,wCard);
	}
}

WORD _stdcall DVAVfwSetCC(WORD wCard, BOOL fCCOn, HWND hwndClient, UINT Msg, DWORD dwId)
{
	unsigned long ulError;
	WORD retval = 0;

	BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_WORD,DTC_WORD,DTC_DWORD,DTC_PTR,DTC_END};
	if (!ver) {
		WORD wCCOn = (WORD)fCCOn;
		if (!pDVAVfwSetCC) return 0;
		return (pDVAVfwSetCC(wCard, wCCOn, hwndClient, Msg, (WORD)dwId));
	} else {
		if (!Invoke16Func) return FALSE;
		Invoke16Func(40,&ulError,parms,wCard,(WORD)fCCOn,hwndClient,(WORD)Msg,dwId,&retval);

		return retval;
	}
}

WORD _stdcall DVATuneGetLockStatus(WORD wCard, WORD wChannel)
{
	unsigned long ulError;
	WORD retval = 0xFFFF;

	BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_PTR,DTC_END};
	if (!ver) {
		if( !pDVATuneGetLockStatus) 
			return( TUNER_LOCK_UNKNOWN);
		return (pDVATuneGetLockStatus(wCard, wChannel));
	} else {
		if (!Invoke16Func) return (TUNER_LOCK_UNKNOWN);
		Invoke16Func(41,&ulError,parms,wCard,wChannel,&retval);
		return retval;
	}
}

DWORD _stdcall DVAAudioDetectStatus(WORD wCard)
{
	unsigned long ulError;
	DWORD retval = 0;

	BYTE parms[] = {DTC_WORD,DTC_PTR,DTC_END};
	if (!ver) {
		if( !pDVAAudioDetectStatus ) return 0;
		return pDVAAudioDetectStatus(wCard);
	} else {
		if (!Invoke16Func) return 0;
		Invoke16Func(38,&ulError,parms,wCard,&retval);
		return retval;
	}
}

DWORD _stdcall DVAAudioSetChannel(WORD wCard, DWORD dwMode)
{
	unsigned long ulError;
	DWORD retval = 0;

	BYTE parms[] = {DTC_WORD,DTC_DWORD,DTC_PTR,DTC_END};
	if (!ver) {
		if( !pDVAAudioSetChannel ) return 0;
		return pDVAAudioSetChannel(wCard, dwMode);
	} else {
		if (!Invoke16Func) return 0;
		Invoke16Func(39,&ulError,parms,wCard,dwMode,&retval);
		return retval;
	}
}

void _stdcall VidSetZoom32(HANDLE hwnd, RECT *lpRect)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_WORD,
					DTC_WORD,DTC_WORD,DTC_END};

	if (ver && Invoke16Func) {
		// Be very careful here ! Make sure this matches the parameters to this method!
		// otherwise you will crash the 16 bit DLL
		Invoke16Func(30,&ulError,parms,hwnd,
				lpRect->left,lpRect->top,
				lpRect->right,lpRect->bottom);
	} else VidSetZoom (hwnd,lpRect);
}

BOOL _stdcall VidSetVBI32(HANDLE hwnd, VBI_STATE state, VBIFAILURE *lpFailure)
{
	// Error return code (not used in this method, but we still must pass it)
	unsigned long ulError;
	WORD retval = 0;

	// Make sure these match exactly the parameters to the DrawShape() function in DummyDLL
	BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_PTR,DTC_PTR,DTC_PTR,DTC_END};

	// Be very careful here ! Make sure this matches the parameters to this method!
	// otherwise you will crash the 16 bit DLL
	if (!Invoke16Func) return FALSE;
	Invoke16Func(36,&ulError,parms,hwnd, (WORD)state,
					(LPBYTE)(lpFailure->lpszFailureString),&(lpFailure->wStringLength),
					&retval);

	return (BOOL)(retval);
}

WORD _stdcall DVAAudioDrivers(void)
{
	unsigned long ulError;
	WORD retval = 0;

	BYTE parms[] = {DTC_PTR,DTC_END};

	if (!Invoke16Func) return 0;

	Invoke16Func(42,&ulError,parms,&retval);

	return retval;
}

BOOL _stdcall DVAAudioEnumDrivers(WORD wAVDriver, LPSTR lpName)
{
	unsigned long ulError;
	WORD retval = 0;

	BYTE parms[] = {DTC_WORD,DTC_PTR,DTC_PTR,DTC_END};

	if (!Invoke16Func) return 0;

	Invoke16Func(43,&ulError,parms,wAVDriver,lpName,&retval);

	return (BOOL)retval;
}

WORD _stdcall DVAAudioConnector(WORD wAVDriver)
{
	unsigned long ulError;
	WORD retval = 0xFFFF;

	BYTE parms[] = {DTC_WORD,DTC_PTR,DTC_END};

	if (!Invoke16Func) return 0xFFFF;

	Invoke16Func(44,&ulError,parms,wAVDriver,&retval);

	return retval;
}

BOOL _stdcall DVAAudioEnumConnector(WORD wAVDriver, WORD wConnector, LPSTR lpName)
{
	unsigned long ulError;
	WORD retval = 0;

	BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_PTR,DTC_PTR,DTC_END};

	if (!Invoke16Func) return 0;

	Invoke16Func(45,&ulError,parms,wAVDriver,wConnector,lpName,&retval);

	return (BOOL)retval;
}

DWORD _stdcall DVAAudioSetConnector(WORD wAVDriver, WORD wConnector, WORD wRecLevel)
{
	unsigned long ulError;
	DWORD retval = 0;

	BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_WORD,DTC_PTR,DTC_END};

	if (!Invoke16Func) return 0;

	Invoke16Func(46,&ulError,parms,wAVDriver,wConnector,wRecLevel,&retval);

	return retval;
}

WORD _stdcall DVAAudioGetRecordLevelConnector(WORD wAVDriver, WORD wConnector)
{
	unsigned long ulError;
	WORD retval = 0xFFFF;

	BYTE parms[] = {DTC_WORD,DTC_WORD,DTC_PTR,DTC_END};

	if (!Invoke16Func) return 0xFFFF;

	Invoke16Func(47,&ulError,parms,wAVDriver,wConnector,&retval);

	return retval;
}
